from .AbstractAPI import AbstractAPI
from .errors import QPidError, TBidError
from .utils import handle_response
from .globals import PLACE_HOLDER


class RG(AbstractAPI):
    """
    The main API interface for the module
    
    Creates new RG object, raises ``AuthError`` if header is invalid

    :type header: dict
    :param header: Auth header obtained from rankguru

    """
    def __init__(self, header: dict):
        super().__init__(
            header,            
            nova = f'https://rest.rankguru.com/tests?textbook={PLACE_HOLDER}',
            graphql = 'https://api.rankguru.com/graphql'
        )

    def get_ans(self, QUESTION_PAPER_ID:str):
    
        """
        This function return a sorted dictionary of answers for the given Question Paper

        :param QUESTION_PAPER_ID: The Question Paper Id of the exam
        :type QUESTION_PAPER_ID: str

        :raises QPidError: If QPid is invalid
        :raises AuthError: If the header is invalid

        :return: Returns a sorted dictionary of question number - answer pairs
        :rtype: dict
        """
        
        r = self.get_ans_raw (QUESTION_PAPER_ID)['data']['QuestionEvaluation']

        if not r:
            raise QPidError
        r = r['evaluatedData']

        ret = {}

        for i in r:
            ret[int (i['questionNo'])] = " ".join ([chr (ord (i) + 48) for i in i['key']])

        keys = [*ret.keys()]
        keys.sort()

        return {i:ret[i] for i in keys}

    def get_ans_raw(self, QUESTION_PAPER_ID:str):
        """
        Returns raw json output from API retrieval

        :param QUESTION_PAPER_ID: The Question Paper Id of the exam
        :type QUESTION_PAPER_ID: str

        :raises AuthError: If the header is invalid
        
        :return: Raw json as python dict
        :rtype: dict
        """
        ANS_QUERY = '{ QuestionEvaluation(input: {questionPaperId: "'+QUESTION_PAPER_ID+'" }) \
                        { evaluatedData { questionNo  key }  } }'
        r = self.handler.make ("graphql", data={'query':ANS_QUERY})
        handle_response(r.status_code)
        return r.json ()

    def get_tests_raw(self, TEXT_BOOK_ID:str):
        """
        Returns raw json output from API retrieval

        :param TEXT_BOOK_ID: The Textbook Id of the category
        :type TEXT_BOOK_ID: str
        
        :raises AuthError: If the header is invalid
        
        :return: Raw json as python dict
        :rtype: dict
        """

        r = self.handler.make ("nova", replace={PLACE_HOLDER:str(TEXT_BOOK_ID)},mode="get")
        handle_response(r.status_code)
        return r.json ()

    def get_tests(self, TEXT_BOOK_ID:str, latest_first = True):
        """
        This function return a sorted dictionary of tests for the given Textbook

        :param TEXT_BOOK_ID: The Textbook Id of the exam
        :type TEXT_BOOK_ID: str
        
        :param latest_first: Toggle it to get earlier tests first, defaults to True
        :type latest_first: bool, optional
        
        :raises TBidError: If TBid is invalid
        :raises AuthError: If the header is invalid

        :return: Returns a dictionary of test name - QPid pairs
        :rtype: dict
        """

        r = self.get_tests_raw (TEXT_BOOK_ID)

        if 'STATUS' in r.keys ():
            raise TBidError

        keys = [*r.keys ()]

        if latest_first:
            keys.reverse ()

        ret = {}
        for i in keys:
            test = r[i]

            ret[test['testName']] = test['questionPaperId']

        return ret



from requests.api import head
from .AbstractAPI import AbstractAPI
from .utils import handle_response

class UserAPI(AbstractAPI):
    
    def __init__(self, header):
        super().__init__(
            header,
            data="https://api.rankguru.com/graphql"
        )


    def get_data_raw(self, scs):
        user_data_query = {"query":"{ StudentById(studentId: \"SCS|\") {\
            studentName, studentId,hierarchyLevels,orientation}\
            }".replace("|",str(scs))
        }

        r = self.handler.make(
            "data", data=user_data_query
        )
        print(r.status_code)
        handle_response(r.status_code)
        # TODO: proper exception

        return r

    def get_data(self, scs):
        data = self.get_data_raw(scs).json()['data']['StudentById']

        if not data:
            raise Exception
            # TODO: PROPER EXCEPTIONS

        del data['hierarchyLevels']['L_6']
        data['data'] = " ".join([*data['hierarchyLevels'].values()])
        del data['hierarchyLevels']
        return data


from .globals import PLACE_HOLDER
from requests import post

def handle_response(code):
    if code in [200,201]:
        return
    else:
        raise {401: Exception,500:ValueError}[code]

def get_header(scs, password):
    r=post(url="https://accounts.rankguru.com/auth/local",data={"email":f"scs{scs}","rememberMe":False,"hostname":"rankguru.com",
                "password":f"{password}","forceLogin":True}).json()
    
    del r["firstTimePasswordChanged"], r["redirectionLink"]

    r["Authorization"]=r["token"]
    r["accesscontroltoken"]=r["accessControlToken"]

    del r["token"],r["accessControlToken"]
    return r

def edit_header(API, header):
    API.handler.header = header


from requests import post, get
from .errors import AuthError


def verify_header(header):
    return get ('https://rest.rankguru.com/tests?textbook=1', headers=header).status_code != 401


class handler:
    def __init__(self, **args):
        self.headers = {}
        self.urls = args

    def set_header(self,header):
        self.headers = header
        if not verify_header (header):
            raise AuthError ()

    def make(self, url_key, data={}, params={}, replace={}, mode='post'):
        url = self.urls[url_key]
        for i in replace:
            url = url.replace(i, replace[i])

        return {
            'post': post,
            'get': get
        }[mode](url, headers=self.headers,
                data=data, params=params)



from .AbstractError import AbstractError


class AuthError (AbstractError):
    """
    Occurs when API header is invalid
    """

    def __init__(self):
        super().__init__('The given Auth has expired or is invalid')


class QPidError (AbstractError):

    """
    Occurs when entered QPid is invalid
    """

    def __init__(self):
        super().__init__('The given Question Paper ID is invalid')


class TBidError (AbstractError):

    """
    Occurs when entered TBid is invalid
    """

    def __init__(self):
        super().__init__('The given Text Book ID is invalid')


class UserDoesNotExist (AbstractError):
    """
    Occurs when the user mentioned doesnt exist
    """

    def __init__(self):
        super().__init__("The User doesnt exist")


class IncorrectPassword (AbstractError):
    """
    Occurs when the given password is incorrect
    """

    def __init__(self):
        super().__init__("The given password is inccorect")



from distutils.core import setup
setup(
  name = 'rankguru',
  packages = ['rankguru'],
  version = '2',
  license='MIT',
  description = 'This is a basic Wrapper for Test answer retrieval from https://rankguru.com',
  author = 'Dapro',
  author_email = 'swaminathshhiju@gmail.com',
  url = 'https://github.com/Dapro9706/rankguru',
  download_url = 'https://github.com/Dapro9706/rankguru/archive/refs/tags/v2.tar.gz',
  keywords = ['Basic'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'requests',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.7'
  ],
)